require 'httparty'

module TT2
  class API
    include HTTParty
    format :json

    attr_accessor :player_id, :auth_token, :created_at
    attr_writer :ad_id

    base_uri 'https://tt2.gamehivegames.com'
    headers 'Accept'       => 'application/vnd.gamehive.btb4-v1.0+json',
            'User-Agent'   => 'BestHTTP',
            'Content-Type' => 'application/json; charset=UTF-8'

    def ad_id
      @ad_id || '00000000-0000-0000-0000-000000000000'
    end

    def query_params(body = '', overrides: {})
      { d:     'ios',
        v:     GAME_VERSION,
        time:  TT2::Util.current_time_ticks,
        dummy: Time.now - @created_at,
        s:     TT2::Util.request_signature(body, ad_id: ad_id) }.merge(overrides)
    end

    def request_body(params = {})
      base = { ad_id:     ad_id,
               player_id: player_id }
      JSON.generate(base.merge(params))
    end

    def reset_time
      @created_at = Time.now
    end

    def get(url)
      self.class.get(url)
    end

    def post(url, body_params = {})
      body = request_body(body_params)
      post_opts = {
        query:   query_params(body),
        body:    body,
        headers: {
          'X-HTTP-Method-Override' => 'POST',
          'Authorization'          => "token #{auth_token}"
        }
      }
      self.class.post(url, post_opts)
    end
  end
end
