require 'openssl'
require 'json'

module TT2
  # Initial Ruby save-file parser from
  # https://www.reddit.com/r/TapTitans2/comments/5zzhx1/clan_mgmt_101_import_save_twice_a_day_kick/df37xnv/
  # modified to clean up a bit.
  module SaveParser
    ENCRYPTION_KEY = ['4bc07927192f4e9a'].pack('H*')

    class << self
      def read(filename = 'ISavableGlobal.adat')
        encrypted = File.read(filename).bytes

        plain = decrypt(encrypted)

        json = plain.slice(plain.index('saveString') + 12...
                           plain.index('playerData') - 2)

        parse_json(json)
      end

      def decrypt(encrypted)
        iv = encrypted.shift(8).pack('c*')
        encrypted = encrypted.pack('c*')

        des = OpenSSL::Cipher.new('DES-CBC')
        des.decrypt
        des.key = ENCRYPTION_KEY
        des.iv = iv

        des.update(encrypted) + des.final
      end

      def parse_json(json)
        # For stripping the JSON of the .NET-special fields `$content` and `$type`
        clean_up = proc do |o|
          next unless o.is_a? Hash
          o.each_pair do |k, v|
            next unless v.is_a? Hash
            v.delete('$type')
            o[k] = v['$content'] if v.key?('$content')
          end
        end

        JSON.load(JSON.load(json), clean_up) # rubocop:disable Security/JSONLoad
      end
    end
  end
end
