require 'openssl'

module TT2
  # This is where all of the fun utility functions like to come and hang out.
  module Util
    module_function

    BOSSES = %w[  Sponkinack Lucanus Caratacos Sklatborg Drakon
                  Straton Grokstomp Grekksfist Zeksgrat FumpTorm
                  RamBloggs Brongunk Plogriftus Darmbonack Gredmash
                  Skurrfesto Ragnosplock Grombdiblort Frombknock Braakstoof
                  Furrglork Skernuut Drezzgart Gelklore Verduug  ].freeze

    PETS = {  'Pet1'  => 'Nova',   'Pet2'  => 'Toto',    'Pet3'  => 'Cerberus',  'Pet4'  => 'Mousy',
              'Pet5'  => 'Harker', 'Pet6'  => 'Bubbles', 'Pet7'  => 'Demos',     'Pet8'  => 'Tempest',
              'Pet9'  => 'Basky',  'Pet10' => 'Scraps',  'Pet11' => 'Zero',      'Pet12' => 'Polly',
              'Pet13' => 'Hamy',   'Pet14' => 'Phobos',  'Pet15' => 'Fluffers',  'Pet16' => 'Kit' }.freeze

    ARTIFACTS = { 'Artifact1'  => 'Heroic Shield',            'Artifact2'  => 'Stone of the Valrunes',
                  'Artifact3'  => 'The Arcana Cloak',         'Artifact4'  => 'Axe of Muerte',
                  'Artifact5'  => "Invader's Shield",         'Artifact6'  => 'Elixir of Eden',
                  'Artifact7'  => 'Parchment of Foresight',   'Artifact8'  => "Hunter's Ointment",
                  'Artifact9'  => "Laborer's Pendant",        'Artifact10' => 'Bringer of Ragnarok',
                  'Artifact11' => "Titan's Mask",             'Artifact12' => 'Swamp Gauntlet',
                  'Artifact13' => 'Forbidden Scroll',         'Artifact14' => 'Aegis',
                  'Artifact15' => 'Ring of Fealty',           'Artifact16' => 'Glacial Axe',
                  'Artifact17' => "Hero's Blade",             'Artifact18' => 'Egg of Fortune',
                  'Artifact19' => 'Chest of Contentment',     'Artifact20' => 'Book of Prophecy',
                  'Artifact21' => 'Divine Chalice',           'Artifact22' => 'Book of Shadows',
                  'Artifact23' => 'Helmet of Madness',        'Artifact24' => 'Staff of Radiance',
                  'Artifact25' => 'Lethe Water',              'Artifact26' => 'Heavenly Sword3',
                  'Artifact27' => 'Glove of Kuma',            'Artifact28' => 'Amethyst Staff',
                  'Artifact29' => 'Drunken Hammer',           'Artifact31' => 'Divine Retribution',
                  'Artifact32' => 'Fruit of Eden',            'Artifact33' => 'The Sword of Storms',
                  'Artifact34' => 'Charm of the Ancient',     'Artifact35' => 'Blade of Damocles',
                  'Artifact36' => 'Infinity Pendulum',        'Artifact37' => 'Oak Staff',
                  'Artifact38' => 'Furies Bow',               'Artifact39' => 'Titan Spear' }.freeze

    AVATARS = {}.freeze

    # The signature hash for all requests
    def request_signature(data, ad_id: '00000000-0000-0000-0000-000000000000')
      key = ad_id[1] + ad_id[0] + ad_id[3]
      digest = OpenSSL::Digest.new('md5')
      OpenSSL::HMAC.hexdigest(digest, key, data)
    end

    # TT2 uses the C# value of DateTime.Now.Ticks, this is the ruby equivalent
    def current_time_ticks
      min_time = Time.utc(0o001, 0o1, 0o1).to_r
      now = Time.now.to_r

      time_delta_nanoseconds = (now - min_time) * Rational(10_000_000)
      time_delta_nanoseconds.ceil
    end

    # The current formula used for Titan Lord HP on the server.
    def titan_lord_hp(cq_level)
      pre_650_factor = 9 * ([cq_level - 1, 650].min**0.9)
      post_650_factor = [(cq_level - 1).to_f / 650, 1].max**1.4

      hp = (10**5) * (15 + pre_650_factor * post_650_factor)
      hp.floor
    end
  end
end
